/*
 * Ethnocentricity Model
 *
 * version m3.1
 *
 */

package edu.brook.ethno10;

import java.awt.*;
import java.util.*;

import edu.brook.ascape.model.*;
import edu.brook.ascape.rule.*;

import edu.brook.ascape.util.*;
import edu.brook.ascape.view.*;

/**
 * A cell in the ethnocentricity lattice.
 *
 **/

public class EthnoCell extends HostCell {
	
	
	//constants
	protected final static int BLUE = 0;
	protected final static int RED = 1;
    protected final static int GREEN = 2;
    protected final static int YELLOW = 3;
    
    protected final static int ALLC = 0;
    protected final static int ALLD = 1;
    protected final static int ETHNO = 2;
    protected final static int ANTIETHNO = 3;

    protected final static int EMPTY = 4;
    
    //agent fields
    protected int[] lastOccupant = new int[2];
    protected boolean hasHadOccupant = false;
    
    private int occupantType = 4;
	//record keeping
	
	    //(list order for arrays as above (see constants))
	
	protected int[] ALLCtoSame = new int[4];
	protected int[] ALLCtoDiff = new int[4];
	protected int[] ALLDtoSame = new int[4];
	protected int[] ALLDtoDiff = new int[4];
	protected int[] ETHNOtoSame = new int[4];
	protected int[] ETHNOtoDiff = new int[4];
	protected int[] ANTIETHNOtoSame = new int[4];
	protected int[] ANTIETHNOtoDiff = new int[4];
	
	public void scapeCreated() {
	    super.scapeCreated();
	}
	
	
	public void initialize() {
		super.initialize();
		
        lastOccupant[0] = EMPTY;
        lastOccupant[1] = EMPTY;
        hasHadOccupant = false;
 
        
        for (int i=0; i<4; i++) {
            ALLCtoSame[i] = 0;
            ALLCtoDiff[i] = 0;
            ALLDtoSame[i] = 0;
            ALLDtoDiff[i] = 0;
            ETHNOtoSame[i] = 0;
            ETHNOtoDiff[i] = 0;
            ANTIETHNOtoSame[i] = 0;
            ANTIETHNOtoDiff[i] = 0;
        }
        
	}
    
    
    public void updateCell() {
    /*
        if (getIteration() >= 1900) { //1900
    
        boolean occupantStrategyOwn = ((EthnoPlayer) this.getOccupant()).getStrategyOwn();
        boolean occupantStrategyOther = ((EthnoPlayer) this.getOccupant()).getStrategyOther();
    
        int occupantTag = ((EthnoPlayer) this.getOccupant()).getTag();
  
        
        if (this.isAvailable()) {
            System.out.println("UPDATING EMPTY CELL!");
        }
        
        
        if (occupantStrategyOwn) {
            if (occupantStrategyOther) {
                occupantType = ALLC;
            }
            else {
                occupantType = ETHNO;
            }
        }//end if strategyOwn
        else {
            if (occupantStrategyOther) {
                occupantType = ANTIETHNO;
            }
            else {
                occupantType = ALLD;
            }
        }//end else

        
        
       // System.out.print("Current Occupant type: ");
       // System.out.println(occupantType);
        
        //UPDATE STATS
        
        if (this.hasHadOccupant) {
        
            if (lastOccupant[0] == occupantTag) { //same color
            
                switch (lastOccupant[1]) {        
                    case ALLC:
                        ALLCtoSame[occupantType]++;
                        break;
                    case ALLD:
                        ALLDtoSame[occupantType]++;
                        break;
                    case ETHNO:
                        ETHNOtoSame[occupantType]++;
                        break;
                    case ANTIETHNO:
                        ANTIETHNOtoSame[occupantType]++;
                        break;
                    default:
                        System.out.println("Error in UpdateStats");
                        break;
                }            
            }
            else { //diff colors
        
                switch (lastOccupant[1]) {    
                    case ALLC:
                        ALLCtoDiff[occupantType]++;
                        //System.out.print("UPDATE ALLCDIFF   ");
                        //System.out.print(getIteration());
                        //System.out.print("   ");
                        //System.out.println(this.getCoordinate());
                        break;
                    case ALLD:
                        ALLDtoDiff[occupantType]++;
                        break;
                    case ETHNO:
                        ETHNOtoDiff[occupantType]++;
                        break;
                    case ANTIETHNO:
                        ANTIETHNOtoDiff[occupantType]++;
                        break;
                    default:
                        System.out.println("Error in UpdateStats");
                        break;
                }            

            }
        }
        else {   
            this.hasHadOccupant = true;
        }
        
        
        //UPDATE LASTOCCUPANT RECORDS
        
        this.lastOccupant[0] = occupantTag;
        this.lastOccupant[1] = occupantType;
        }
      */  
     }
 

 public int getALLCtoDiffALLD() {
    return (this.ALLCtoDiff[ALLD]);
 }
 public int getOccupantType() {
    return (this.occupantType);
 }
 public int getHasHadOccupant() {
    if (this.hasHadOccupant) {
        return (1);
    }
    else {
        return (0);
    }
 }
 
 public int getALLCtoSame(int type) {
    return (this.ALLCtoSame[type]);
 }
 public int getALLDtoSame(int type) {
    return (this.ALLDtoSame[type]);
 }
 public int getETHNOtoSame(int type) {
    return (this.ETHNOtoSame[type]);
 }
 public int getANTIETHNOtoSame(int type) {
    return (this.ANTIETHNOtoSame[type]);
 }
 
 public int getALLCtoDiff(int type) {
    return (this.ALLCtoDiff[type]);
 }
 public int getALLDtoDiff(int type) {
    return (this.ALLDtoDiff[type]);
 }
 public int getETHNOtoDiff(int type) {
    return (this.ETHNOtoDiff[type]);
 }
 public int getANTIETHNOtoDiff(int type) {
    return (this.ANTIETHNOtoDiff[type]);
 }
     
     
}