 /*
 * PLAYER CODE 
 * Model of Discriminatory Altruism
 * (variant of Ethno model)
 *
 * Version m3.1
 *
 * This code created originally on 1/21/03
 * (copied from package "ethnoadd" and modified)
 *
 * Additions:
 *
 */

package edu.brook.ethno10;

import java.awt.*;
import java.util.*;

import edu.brook.ascape.model.*;
import edu.brook.ascape.rule.*;

import edu.brook.ascape.util.*;
import edu.brook.ascape.view.*;

/**
 * A player in the ethnocentricity model.
 *
 **/

public class EthnoPlayer extends CellOccupant   {
	
	
	//constants
	protected final static int BLUE = 0;
	protected final static int RED = 1;
    protected final static int GREEN = 2;
    protected final static int YELLOW = 3;
    
    protected final int MEETOWN = 0;
	protected final int MEETOTHER = 1;
	protected final int DEFOTHER = 2;
	protected final int COOPOWN = 3;
    protected final int COOPOTHER = 4;
    
    //agent fields
    protected int tag;
	protected boolean strategyToOwn;  //TRUE = COOPERATE
	protected boolean strategyToOther;  //TRUE = COOPERATE
		
    protected double currOTR;

	protected boolean unplaced = false;
	protected CoordinateDiscrete parentLoc = null;
	protected boolean justBorn = false;
	
	protected int fitnessScore = 0;
	
	protected int founderXLoc;
	protected int founderYLoc;
	protected int shadowTag;
	protected boolean shadowStrategyToOwn;
	protected boolean shadowStrategyToOther;

	
	//INITIALIZE AN AGENT (clone or immigrant)	
	public void initialize() {
		super.initialize();
   
        //check for OneTagOnly
        if ((((EthnoModel) scape.getModel()).getOneTagOnly())) {
            tag = BLUE;
        }
        else {
            if (((EthnoModel) scape.getModel()).getUseMoreColors()) {
         		int numColors = ((EthnoModel) scape.getModel()).getNumColors();
                tag = randomInRange(0,(numColors-1));    //0,3 normal
            }
            else {
                if (randomIs()) {
                    tag = BLUE;
                }
                else {
                    tag = RED;
                }
            }
        }
              
        strategyToOwn = randomIs();
        strategyToOther = randomIs();
        //strategyToOther = strategyToOwn;  //for an initial condition of all CCs and DDs

		this.shadowTag = this.tag;
		this.shadowStrategyToOwn = this.strategyToOwn;
		this.shadowStrategyToOther = this.strategyToOther;
		
	}
    
    
    
	public void scapeCreated() {
		
		getScape().addInitialRule(MOVE_RANDOM_LOCATION_RULE);
	    getScape().addRule(UPDATE_RULE); //Resets OTR at beginning of round

        
        //The code below adds various "play" or interaction rules
        //Only one of the three rules below should be active (the
        //other two should be commented out)

        //USE THIS RULE FOR "REGULAR CASE" 
			getScape().addRule(PLAY_NEIGHBORS_RULE); //play all neighbors
        
        //USE THIS RULE FOR RANDOM LOCATION VARIANT                
   		/*     
        getScape().addRule(new Rule("Play a Random Partner") {       
            public void execute(Agent a) {
                 	for (int i=0; i<4; i++) {
                 		playRandomGlobal(a);
                    }
            }
        });
   		*/
        
        
        //USE THIS RULE FOR PLAY ONE NEIGHBOR ONLY VARIANT
        /*
        getScape().addRule(new Rule("Play a Random Neighbor") {
		    public void execute(Agent a) {
		        Cell[] neighbors = ((EthnoPlayer) a).getHostCell().getNeighbors();
		        
		        boolean neighborAvail = false;       
		        for (int i=0; i<neighbors.length; i++) {
		            if (!(neighbors[i].isAvailable())) {
		                neighborAvail = true;
		            }
		        }
		        if (neighborAvail) {
		            boolean done = false;
		            while (!(done)) {
		                int pick = randomToLimit(neighbors.length);
		                if (!(neighbors[pick].isAvailable())) {
		                    done = true;
		                    EthnoPlayer partner = ((EthnoPlayer) (neighbors[pick].getOccupant()));
		                    ((EthnoPlayer) a).play(partner);
		                }
		            }
		        }
		    }
		});
		*/
	
		getScape().addRule(FISSIONING_RULE); //This rule handles cloning

	}
    
    
    public void playRandomGlobal(Agent a) { //Active only if specified above
        
        EthnoPlayer candidatePartner = null;
		do {
		    //Find random candidate uniformally distributed across all agents
			int index = randomToLimit(scape.getSize());
			candidatePartner = (EthnoPlayer) ((ScapeVector) scape).getCell(index);
		} while ((candidatePartner == this));
		
		if (candidatePartner.getHostCell().isAvailable()) {
		    System.out.println("EMPTY PARTNER!!!");
		}
		
		
        a.play(candidatePartner);
        
    }
    
    
    public void update() { //RESET OTR TO BASE RATE
    
        this.currOTR = ((EthnoModel) scape.getModel()).getBaseOTR(); 
        
        if (((EthnoModel) scape.getModel()).getUseDiscrimCost()) { 
        	if (!(strategyToOwn == strategyToOther)) {
        		this.currOTR -= ((EthnoModel) scape.getModel()).getDiscrimCost();
        	}        	
        }  
   
   		scape.getData().getStatCollector("Ethnocentrics").addValueFor(this);
   		scape.getData().getStatCollector("DefectAgainstAll").addValueFor(this);
   		scape.getData().getStatCollector("Reds").addValueFor(this);
   		scape.getData().getStatCollector("Blues").addValueFor(this);
   		scape.getData().getStatCollector("Greens").addValueFor(this);
   		scape.getData().getStatCollector("Yellows").addValueFor(this);
   		scape.getData().getStatCollector("RedDiscrim").addValueFor(this);
   		scape.getData().getStatCollector("BlueDiscrim").addValueFor(this);
   		scape.getData().getStatCollector("PurpleDiscrim").addValueFor(this);
   		scape.getData().getStatCollector("YellowDiscrim").addValueFor(this);
   		
   		scape.getData().getStatCollector("CoopBoth").addValueFor(this);
   		scape.getData().getStatCollector("CoopOwnDefOther").addValueFor(this);
   		scape.getData().getStatCollector("DefOwnCoopOther").addValueFor(this);
   		scape.getData().getStatCollector("DefBoth").addValueFor(this);
   		
   		
   		this.justBorn = false;
   		this.fitnessScore = 0;
   		
   		
   		if (getIteration() == 1) {
   		
        this.founderXLoc = ((Coordinate2DDiscrete) this.getHostCell().getCoordinate()).getXValue();
        this.founderYLoc = ((Coordinate2DDiscrete) this.getHostCell().getCoordinate()).getYValue();
  
   		}
   		
   		/*
   		if (getIteration() == ((EthnoModel) scape.getModel()).getStopIteration()) {
   			((EthnoModel) scape.getModel()).addToRegressionList(this.getCoordinate());	
   		}
   		*/
    }
   
   
    public boolean fissionCondition() {//FLIP BASED ON OTR FOR CLONING CHANCE
            
       if (randomInRange(0.0,1.0) < this.currOTR) {
            return (true);
       }
       else {
            return (false);
       }
        
    }
    
    
      
    public void fission() { //CLONING ROUTINE (active if fired by FissionCondition)
                              
          //create offspring
          EthnoPlayer child = (EthnoPlayer) this.clone();
          //((ScapeVector) ((EthnoModel) scape.getModel()).getUnplacedList()).addAgent(child);
          ((ScapeVector) getScape()).addAgent(child);
          
          child.unplaced = true;
          child.parentLoc = this.getCoordinate();
     	  child.founderXLoc = this.founderXLoc;
     	  child.founderYLoc = this.founderYLoc;
          
          //mutate strategyToOwn
          if (getRandom().nextFloat() < ((EthnoModel) scape.getModel()).getMutationRate()) {
              child.strategyToOwn = !(this.strategyToOwn);
          }
          else {
          	child.strategyToOwn = this.strategyToOwn;
          }
                
                
		  //mutate strategyToOther?
          if (getRandom().nextFloat() < ((EthnoModel) scape.getModel()).getMutationRate()) {
             child.strategyToOther = !(this.strategyToOther);    
          }
          else {
             child.strategyToOther = this.strategyToOther;
          }
                
          //mutate tag?
          if ((((EthnoModel) scape.getModel()).getOneTagOnly())) { //ONE COLOR CASE
             child.tag = this.tag;
          }
          else {//MULTIPLE COLORS
             if (getRandom().nextFloat() < ((EthnoModel) scape.getModel()).getMutationRate()) {    
                                
                        if (((EthnoModel) scape.getModel()).getUseMoreColors()) {
                
                			int numColors = ((EthnoModel) scape.getModel()).getNumColors();
                		    child.tag = randomInRange(0,(numColors-1)); //0,3 normal //0,7 for 8 colors

                            while (child.tag == this.tag) {
                                child.tag = randomInRange(0,(numColors-1)); //0,3 normal  //0,7 for 8 colors
                            }
                            
                        }
                        else { //2 colors
                            if (this.tag == BLUE) {child.tag = RED;}                    
                            else {child.tag = BLUE;}
                                       
                        }
                    }
                    else {child.tag = this.tag;} //Don't mutate
                }
                
        
        
        //mutation of shadow alleles calculated with separate random draws
        if (getRandom().nextFloat() < ((EthnoModel) scape.getModel()).getMutationRate()) {
            child.shadowStrategyToOwn = !(this.shadowStrategyToOwn);
        }
        else {
          	child.shadowStrategyToOwn = this.shadowStrategyToOwn;
        }
        
        if (getRandom().nextFloat() < ((EthnoModel) scape.getModel()).getMutationRate()) {
            child.shadowStrategyToOther = !(this.shadowStrategyToOther);    
        }
        else {
            child.shadowStrategyToOther = this.shadowStrategyToOther;
        }
        
        if ((((EthnoModel) scape.getModel()).getOneTagOnly())) {child.shadowTag = this.shadowTag;}
        else {//MULTIPLE COLORS
             if (getRandom().nextFloat() < ((EthnoModel) scape.getModel()).getMutationRate()) {    
                                
                        if (((EthnoModel) scape.getModel()).getUseMoreColors()) {
                
                			int numColors = ((EthnoModel) scape.getModel()).getNumColors();
                		    child.shadowTag = randomInRange(0,(numColors-1)); //0,3 normal //0,7 for 8 colors

                            while (child.shadowTag == this.shadowTag) {
                                child.shadowTag = randomInRange(0,(numColors-1)); //0,3 normal  //0,7 for 8 colors
                            }
                            
                        }
                        else { //2 colors
                            if (this.shadowTag == BLUE) {child.shadowTag = RED;}                    
                            else {child.shadowTag = BLUE;}
                                       
                        }
                    }
                    else {child.shadowTag = this.shadowTag;} //Don't mutate
              }
                
        
        
        
        child.justBorn = true;
        
        
    }//end fission()
    
    
    
    public void play(Agent partner) {
        
        boolean strategy = false;
        boolean partnerStrategy = false;
          
                  
        //DECIDE WHICH STRATEGY BIT TO USE
        if (this.tag == ((EthnoPlayer) partner).tag) {  //both the same tag, so use "OWN" strategy
                
                strategy = this.strategyToOwn;
        
                //If there's noise, check to see if triggered to switch strategy bit used
                
                if (((EthnoModel) scape.getModel()).getUseNoise()) {
                    if (randomInRange(0.0,1.0) < ((EthnoModel) scape.getModel()).getNoiseLevel()) {
                        strategy = this.strategyToOther;
                    }
                }    
                
        }
        else { //different tags, use "OTHER" strategy
                
                strategy = this.strategyToOther;
                
                //If there's noise, check to see if triggered to switch strategy bit used
                
                if (((EthnoModel) scape.getModel()).getUseNoise()) {
                    if (randomInRange(0.0,1.0) < ((EthnoModel) scape.getModel()).getNoiseLevel()) {
                        strategy = this.strategyToOwn;
                    }
                }    
                
        }
        
        /////////////////////
        /*
		if ((((EthnoModel) scape.getModel()).getOneTagOnly())) {
            strategy = strategyToOwn;
        }
        */
        ///////////////    
            
        //UPDATE BEHAVIOR STATISTICS
        if (this.tag == ((EthnoPlayer) partner).tag) { //they're same type 
        
            ((EthnoModel) scape.getModel()).incrementPhenotype(MEETOWN);
            
            if (strategy) {
                ((EthnoModel) scape.getModel()).incrementPhenotype(COOPOWN);    
            }
            
        }
        else { //they're different types
        
            ((EthnoModel) scape.getModel()).incrementPhenotype(MEETOTHER);
            
            if (!(strategy)) {
                ((EthnoModel) scape.getModel()).incrementPhenotype(DEFOTHER);
            }
            else {
                ((EthnoModel) scape.getModel()).incrementPhenotype(COOPOTHER);
            }
                
        }
        
        
        //UPDATE OTRs BASED ON GIVING
        
        if (strategy == EthnoModel.COOPERATE) {
            this.currOTR = this.currOTR - ((EthnoModel) scape.getModel()).getCostOfGiving();
            ((EthnoPlayer) partner).currOTR = ((EthnoPlayer) partner).currOTR + ((EthnoModel) scape.getModel()).getGainOfReceiving();
        }
        else {
        	
        	if (((EthnoModel) scape.getModel()).getUseCombat()) {
        		
        		this.currOTR = this.currOTR + ((EthnoModel) scape.getModel()).getCostOfGiving();
        		((EthnoPlayer) partner).currOTR = ((EthnoPlayer) partner).currOTR - ((EthnoModel) scape.getModel()).getGainOfReceiving();	
        		
        	}	
        	
        }
        
    }
    
    
    
	public Color getColor() {
	    
	    //usual color scheme
		if (tag == BLUE) {
			return Color.blue;
		}
		if (tag == RED) {
		    return Color.red;
		}
		if (tag == GREEN) {
		    return Color.green;
		}
		if (tag == YELLOW) {
		    return Color.yellow;
		}
		else {
			return Color.black; //default, if appears means error
		}
    }
    
    
    //GETTERS AND SETTERS
    
    public int getTag() {
        return (this.tag);
    }
    
    public int getStrategyToOwn() {
        if (this.strategyToOwn) {
            return (1); //this means cooperate
        }
        else {
            return (0); //this means defect
        }   
    }
    public void setStrategyToOwn(int _strategy) {
        
    }
    
    public int getShadowStrategyToOwn() {
    	if (this.shadowStrategyToOwn) {return (1);}
        else {return (0);}
    
    }

    public int getShadowStrategyToOther() {
    	if (this.shadowStrategyToOther) {return (1);}
        else {return (0);}
    
    }

	public int getShadowTag() {
		return (this.shadowTag);	
	}
	
	public double getPTR() {
		
		return (this.currOTR);	
	}
    
    public CoordinateDiscrete getParentLoc() {
    	return (this.parentLoc);
    }
     
    public int getStrategyToOther() {
        if (this.strategyToOther) {
            return (1); //this means cooperate
        }
        else {
            return (0); //this means defect
        }   
    }
    public void setStrategyToOther(int _strategy) {
 
    }

	public boolean getJustBorn() {
		return (justBorn);
	}
	
	public int getFitness() {
		return (fitnessScore);
	}
	
	public void incrementFitness() {
		this.fitnessScore++;
	}
	

	public boolean getIsUnplaced() {
		return (unplaced);
	}
	public void setIsUnplaced(boolean _value) {
		this.unplaced = _value;
	}
	
    public boolean getStrategyOwn() {
        return (this.strategyToOwn);
    }
    public boolean getStrategyOther() {
        return (this.strategyToOther);
    }

    
}